@@ -7,16 +7,17 @@ import (
 
 	controllerapi "github.com/docker/buildx/controller/pb"
 	"github.com/docker/buildx/monitor/types"
+	"github.com/pkg/errors"
 )
 
 type RollbackCmd struct {
 	m types.Monitor
 
-	invokeConfig controllerapi.InvokeConfig
+	invokeConfig *controllerapi.InvokeConfig
 	stdout       io.WriteCloser
 }
 
-func NewRollbackCmd(m types.Monitor, invokeConfig controllerapi.InvokeConfig, stdout io.WriteCloser) types.Command {
+func NewRollbackCmd(m types.Monitor, invokeConfig *controllerapi.InvokeConfig, stdout io.WriteCloser) types.Command {
 	return &RollbackCmd{m, invokeConfig, stdout}
 }
 
@@ -37,6 +38,9 @@ COMMAND and ARG... will be executed in the container.
 }
 
 func (cm *RollbackCmd) Exec(ctx context.Context, args []string) error {
+	if ref := cm.m.AttachedSessionID(); ref == "" {
+		return errors.Errorf("no attaching session")
+	}
 	cfg := cm.invokeConfig
 	if len(args) >= 2 {
 		cmds := args[1:]
@@ -47,6 +51,7 @@ func (cm *RollbackCmd) Exec(ctx context.Context, args []string) error {
 		if len(cmds) > 0 {
 			cfg.Entrypoint = []string{cmds[0]}
 			cfg.Cmd = cmds[1:]
+			cfg.NoCmd = false
 		}
 	}
 	id := cm.m.Rollback(ctx, cfg)
